from databricks_cli.jobs.api import JobsApi
from databricks_cli.sdk import ApiClient


class JobsService:

    def __init__(self, host, token):
        self.jobs_api = JobsService._jobs_api(host, token)
        self.all_jobs = self._all_jobs()

    @staticmethod
    def _jobs_api(host, token):
        api_client = ApiClient(host=host, token=token, api_version='2.1')
        return JobsApi(api_client)

    def _all_jobs(self) -> list:
        has_more = True
        jobs = []
        offset = 0
        limit = 25

        while has_more:
            jobs_json = self.jobs_api.list_jobs(offset=offset, limit=limit, version='2.1')
            jobs += jobs_json['jobs'] if 'jobs' in jobs_json else []
            has_more = jobs_json.get('has_more', False)
            if has_more:
                offset = offset + (len(jobs_json['jobs']) if 'jobs' in jobs_json else limit)

        return [Job(job) for job in jobs]

    def list(self, filter:str=None) -> []:
        _jobs = self.all_jobs
        if filter:
            _jobs = [j for j in _jobs if j.contains(filter)]
        return _jobs


class Job():
    def __init__(self, job_dict: dict):
        self.id:str = str(job_dict['job_id'])
        self.name:str = job_dict['settings']['name']
        self.tags:str = Job._tags(job_dict)
        self.creator:str = Job._creator(job_dict)
        self.schedule_status:str = Job._schedule_status(job_dict)

    @staticmethod
    def _tags(job_dict: dict) -> str:
        tags_dict = job_dict['settings'].get('tags', {})
        list = [v or k for (k, v) in tags_dict.items()]

        return ','.join(list)

    @staticmethod
    def _creator(job_dict: dict) -> str:
        try:
            result = job_dict['creator_user_name']
        except KeyError:
            result = ''
        return result

    @staticmethod
    def _schedule_status(job_dict: dict) -> str:
        try:
            result = job_dict['settings']['schedule']['pause_status']
        except KeyError:
            result = ''
        return result

    def contains(self, filter: str) -> bool:
        fl = filter.lower()
        if fl in self.id.lower():
            return True
        if fl in self.name.lower():
            return True
        if fl in self.tags.replace(',', '').lower():
            return True
        if fl in self.creator.lower():
            return True

        return False


import json
from enum import Enum

import requests
from requests import HTTPError


class AccessType(str, Enum):
    USER = 'user'
    PRINCIPAL = 'principal'
    GROUP = 'group'

    def access_name(self):
        _access_names = {AccessType.USER: 'user_name',
                         AccessType.PRINCIPAL: 'service_principal_name',
                         AccessType.GROUP: 'group_name'}
        return _access_names[self]

    def label(self):
        _lables = {AccessType.USER: 'User',
                   AccessType.PRINCIPAL: 'Service principal',
                   AccessType.GROUP: 'Group'}
        return _lables[self]


class Level(str, Enum):
    CAN_VIEW = 'can_view'
    CAN_MANAGE = 'can_manage'
    CAN_MANAGE_RUN = 'can_manage_run'

    def label(self):
        _lables = {Level.CAN_VIEW: 'Can View',
                   Level.CAN_MANAGE: 'Can Manage',
                   Level.CAN_MANAGE_RUN: 'Can Manage Run'}
        return _lables[self]


class Permission():
    def __init__(self, name: str, type: AccessType, level: Level):
        self.name = name
        self.level = level
        self.type = type

    def json(self):
        return {self.type.access_name(): self.name, 'permission_level': self.level.name}


class PermissionsService():
    def __init__(self, host, token):
        self.host = host
        self.token = token

    def assign_permission(self, job_id, permission: Permission):
        url = f"{self.host}/api/2.0/permissions/jobs/{job_id}"
        data = {'access_control_list': [permission.json()]}
        # print(data)

        try:
            response = requests.patch(url=url, auth=('token', self.token), json=data)
            response.raise_for_status()
        except HTTPError as http_error:
            print(json.dumps(json.loads(response.text), indent=2))
            # error_message = response.json()['message']
            # print(error_message)
            raise http_error
        except Exception as error:
            print(f'Other error occured: {error}')
            raise error



#!/usr/bin/env python

import typer
from rich.progress import track, Progress, SpinnerColumn
from tabulate import tabulate

import fish_databricks_jobs.config as config
from fish_databricks_jobs.services.jobs import Job, JobsService
from fish_databricks_jobs.services.permissions import Permission, AccessType, Level, PermissionsService
from fish_databricks_jobs.services.version import package_version

app = typer.Typer(context_settings=dict(help_option_names=['-h', '--help']))


def version_callback(value: bool):
    if value:
        typer.echo(f'Version: {package_version}')
        raise typer.Exit()


@app.callback()
def common(
        version: bool = typer.Option(None, '--version', '-v', callback=version_callback, help=package_version),
):
    pass


@app.command()
def list(
        filter: str = typer.Option(None, '--filter', '-f', help='filter jobs, case insensitively'),
        profile: str = typer.Option('DEFAULT', '--profile', '-p', help='profile name in ~/.databrickscfg')
):
    '''List Databricks jobs in table (id name tags creator schedule_status)
    '''
    with Progress(SpinnerColumn(spinner_name='aesthetic'), transient=True) as progress:
        progress.add_task(description=None, total=None)
        return _list(filter, profile)

def _list(filter, profile):
    host, token = config.get(profile)
    service = JobsService(host, token)
    jobs = service.list(filter)
    size = len(jobs)

    if size == 0:
        print('no job found')
        return jobs, size

    print(tabulate(_jobs_to_table(jobs), tablefmt='plain', disable_numparse=True))
    print(f'size: {size}')
    return jobs, size


@app.command()
def permission_assign(
        name: str = typer.Argument(..., help='User name, group name or serive principal id. Who will receive the permisssion.'),
        type: AccessType = typer.Option(AccessType.USER.value, '--type', '-t', help='Permission receiver type.', case_sensitive=False),
        level: Level = typer.Option(..., '--level', '-l', help='Permission level.', case_sensitive=False),
        filter: str = typer.Option(None, '--filter', '-f', help='filter jobs, case insensitively.'),
        profile: str = typer.Option('DEFAULT', '--profile', '-p', help='profile name in ~/.databrickscfg'),
        force: bool = typer.Option(False, '--force', help='Attempt to assign permission without prompting for confirmation. **Use this flag with caution**')
):
    '''Assign permission to user
    '''
    jobs, size = list(filter=filter, profile=profile)
    if size == 0:
        raise typer.Exit()

    print(f'\nAbout to assign {type.value}({name}) with permission({level.value}) for above {size} job{"s" if size>1 else ""}.')
    if not force:
        typer.confirm(f'Are you sure you want to continue?', abort=True)

    host, token = config.get(profile)
    permission = Permission(name, type=type, level=level)

    service = PermissionsService(host, token)
    for job in track(jobs, description=f'[green]Assigning permission({permission.level.value})...', complete_style='bar.finished'):
        print(job.name)
        try:
            service.assign_permission(job_id=job.id, permission=permission)
        except:
            print('Error occured!')
            raise typer.Exit()

    print(f'\nSucceeded.')

def _jobs_to_table(jobs: [Job]):
    ret = []
    for j in jobs:
        ret.append((j.id, j.name, j.tags, j.creator, j.schedule_status))
    return sorted(ret, key=lambda j: j[1].lower())


if __name__ == '__main__':
    app()



import configparser
from pathlib import Path


def get(profile='DEFAULT'):
    config_file = Path(f'{Path.home()}/.databrickscfg')

    parser = configparser.RawConfigParser()
    parser.read(config_file)

    host = parser.get(profile, 'host')
    token = parser.get(profile, 'token')

    return (host, token)




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fish_databricks_jobs', 'fish_databricks_jobs.services']

package_data = \
{'': ['*']}

install_requires = \
['databricks-cli>=0.17.0,<0.18.0',
 'tabulate>=0.9.0,<0.10.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['fish-databricks-jobs = fish_databricks_jobs.cli:app',
                     'jobser = fish_databricks_jobs.cli:app']}

setup_kwargs = {
    'name': 'fish-databricks-jobs',
    'version': '0.7.5',
    'description': 'cli and sdk to manage Jobs in Databricks',
    'long_description': "# fish-databricks-jobs \n\nfish-databricks-jobs is cli and python sdk to manage Jobs for Databricks. e.g assign permissions to multiple jobs. User can filter jobs by job name or tags.  \n\nThe functionality of current [databricks-cli(v0.17.4)](https://pypi.org/project/databricks-cli/) is limited on the `jobs` api. e.g it can not assign job permission.\n\nWith `fish-databricks-jobs`, you can assign job permission, e.g: \n\nto assign group `mygroup` with permission `can_manage` to job by filter `843966944901662`(job_id) \n```angular2html\n$ fish-databricks-jobs permission-assign mygroup --type group --level can_manage --filter 843966944901662\n```\n# Requirements\n- Python Version >= 3.7 \n# Installation\n```\n$ pip install --upgrade fish-databricks-jobs\n```\n```angular2html\n$ fish-databricks-jobs --version\nVersion: 0.6.8\n```\n# Usage\n### permission-assign\n```\n$ fish-databricks-jobs permission-assign -h\n\n Usage: fish-databricks-jobs permission-assign [OPTIONS] NAME\n\n Assign permission to user\n\n╭─ Arguments ─────────────────────────────────────────────────────────────────────────────────────────────────────────────╮\n│ *    name      TEXT  User name, group name or serive principal id. Who will receive the permisssion. [default: None]    │\n│                      [required]                                                                                         │\n╰─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────╯\n╭─ Options ───────────────────────────────────────────────────────────────────────────────────────────────────────────────╮\n│    --type     -t      [user|principal|group]                Permission receiver type. [default: user]                   │\n│ *  --level    -l      [can_view|can_manage|can_manage_run]  Permission level. [default: None] [required]                │\n│    --filter   -f      TEXT                                  filter jobs, case insensitively. [default: None]            │\n│    --profile  -p      TEXT                                  profile name in ~/.databrickscfg [default: DEFAULT]         │\n│    --force                                                  Attempt to assign permission without prompting for          │\n│                                                             confirmation. **Use this flag with caution**                │\n│    --help     -h                                            Show this message and exit.                                 │\n╰─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────╯\n```\n### use as sdk to list jobs\n```angular2html\nfrom fish_databricks_jobs.services.jobs import JobsService, Job\nhost, token = 'https://example.cloud.databricks.com','exampletokenc0e27d8b91fd8c0144f0a23'\njob_list = JobsService(host, token).list()\ndf = spark.createDataFrame(job_list)\n\ndisplay(df)\n```\n# Config authentication\nfish-databricks-jobs uses same config file as `databricks-cli`. e.g.`~/.databrickscfg` for macOS. Similar for Windows.\n```\n[DEFAULT]\nhost = https://example.cloud.databricks.com\ntoken = exampletokenc0e27d8b91fd8c0144f0a23\n```\n\n\n\n",
    'author': 'Tim Chen',
    'author_email': 'firstim@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/firstim/fish-databricks-jobs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
